/*
        clgrep.h -- Container Lite v 1.89
        (C) Copyright 1995  John Webster Small
        All rights reserved
        Demo: CC -DCLGREP_DEMO clgrep cstr cl
*/

#ifndef clgrep_h
#define clgrep_h

#ifndef cl_h
  #include "cl.h"
#endif

#ifndef cstr_h
  #include "cstr.h"
#endif

#ifndef CL_GREP_MAX_RECUR
  #define CL_GREP_MAX_RECUR 3
#endif

class grep_tok
{ // plays double duty as polycluster root
  // and the "." wildcard
  public:
    enum  occurences
        { ZeroOrOne, One, OneOrMore, Any }; 
    occurences count;
    grep_tok() : count(One) {}
    virtual ~grep_tok();
    virtual int matchOne
        (const char * SS, size_t& len, int caseIgnore,
        CL<char> * groupToks);
    CL_StrongTyping(grep_tok);
    static void error(const char * GRE,
        size_t pos = 0, const char * msg = 0);
    static grep_tok * newTok(const char * GRE,
        size_t GRE_len, size_t& tokLen, int groups);
};

class grep_exp : public CL<grep_tok>
{   // Helper class to grep_subExp and CL_grep
  public:
    grep_exp() : CL<grep_tok>(CL_DEL) {}
    int parse(const char * GRE, size_t GRE_len,
        int groups);
    void parseGroup
        (char leftDelimiter, char rightDelimiter,
        const char * GRE, size_t GRE_len,
        size_t& tokLen, int groups);
    int match(const char * SS,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
};

class grep_subExp : public grep_tok, grep_exp
{
  public:
    grep_subExp(const char * GRE, size_t GRE_len,
        size_t& tokLen, int groups);
    virtual int matchOne(const char * SS,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
};

class grep_group : public grep_tok, grep_exp
{  // groups mark tokens, e.g. {groupToken}
  public:
    grep_group(const char * GRE, size_t GRE_len,
        size_t& tokLen);
    virtual int matchOne(const char * SS,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
};

class CL_grep
{
    grep_exp parseTree;
    cstr gre;
    int BOL_anchor, EOL_anchor, _groups;
  protected:
    int _init(const char * GRE, int groups = 0);
    CL_grep(const char * GRE, int groups);
    int _match(const char * SS, size_t& idx,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
  public:
    CL_grep(const char * GRE = 0);
    virtual ~CL_grep() {}
    virtual int init(const char * GRE);
    int grepOk() { return parseTree.nodes() > 0; }
    virtual int match(const char * SS, size_t& idx,
        size_t& len, int caseIgnore = 0);
    ostream& printOn(ostream& os)
        { return gre.printOn(os); }
    ostream& storeOn(ostream& os);
    virtual istream& readFrom(istream& is);
    friend ostream& operator<<(ostream& os, CL_grep& g)
        { return g.storeOn(os); }
    friend istream& operator>>(istream& is, CL_grep& g)
        { return g.readFrom(is); }
    CL_StrongTypingStream(CL_grep);
};

class CL_ggrep : public CL_grep, public CL<char>
{ // CL<char> contains group tokens matched
  public:
    CL_ggrep(const char * GRE = 0);
    virtual int init(const char * GRE);
    int groupToksOk()
        { return maxNodes() == CL_MAXNODES; }
    virtual int match(const char * SS, size_t& idx,
        size_t& len, int caseIgnore = 0);
    virtual istream& readFrom(istream& is);
    friend ostream& operator<<
        (ostream& os, CL_ggrep& gg)
        { return gg.CL_grep::storeOn(os); }
    friend istream& operator>>
        (istream& is, CL_ggrep& gg)
        { return gg.CL_grep::readFrom(is); }
    CL_StrongTypingStream(CL_ggrep);
};

class grep_seq : public grep_tok, public cstr
{
  public:
    grep_seq(const char * GRE, size_t GRE_len,
        size_t& tokLen, int groups);
    virtual int matchOne(const char * SS,
        size_t& len_, int caseIgnore,
        CL<char> * groupToks);
};

class grep_setTok
{
  public:
    grep_setTok() {}
    virtual ~grep_setTok();
    virtual int match(char c, int caseIgnore);
    CL_StrongTyping(grep_setTok);
};

class grep_setAlts : public grep_setTok, public cstr
{
  public:
    grep_setAlts(const char * SEQ);
    virtual int match(char c, int caseIgnore);
};

class grep_setRange : public grep_setTok
{
    int _left, _right;
  public:
    grep_setRange(int left, int right);
    virtual int match(char c, int caseIgnore);
};

class grep_set : public grep_tok, CL<grep_setTok>
{
    int negate;
  public:
    grep_set(const char * GRE, size_t GRE_len,
        size_t& tokLen);
    virtual int matchOne(const char * SS,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
};

class grep_alt : public grep_tok
{
    grep_tok * _left, * _right;
  public:
    grep_alt(const char * GRE, size_t GRE_len,
        grep_tok * leftTok, size_t& rightTokLen,
        int groups);
    virtual ~grep_alt();
    virtual int matchOne(const char * SS,
        size_t& len, int caseIgnore,
        CL<char> * groupToks);
};

#endif
